/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

import (
	"time"
)

//  Runtime statistics for a container
type Statistics struct {
	Timestamp time.Time `json:"Timestamp,omitempty"`

	ContainerStartTime time.Time `json:"ContainerStartTime,omitempty"`

	Uptime100ns uint64 `json:"Uptime100ns,omitempty"`

	Processor *ProcessorStats `json:"Processor,omitempty"`

	Memory *MemoryStats `json:"Memory,omitempty"`

	Storage *StorageStats `json:"Storage,omitempty"`
}
